from __future__ import annotations

import json
import smtplib
import ssl
from email.message import EmailMessage
from typing import Iterable, List, Optional

import requests

from .email_config import SmtpConfig, NotificationUser


class TelegramTransport:
	"""Транспорт для отправки отчетов через Telegram Bot API."""
	
	def __init__(self, bot_token: Optional[str], users: Optional[Iterable[NotificationUser]]) -> None:
		"""Инициализация Telegram транспорта.
		
		Args:
			bot_token: Токен бота Telegram
			users: Список пользователей с настройками Telegram
		"""
		self.bot_token = bot_token
		self.users = list(users) if users else []

	@property
	def enabled(self) -> bool:
		"""Проверить, настроен ли транспорт для отправки.
		
		Returns:
			bool: True если есть токен и пользователи с Telegram, False иначе
		"""
		return bool(self.bot_token and any(user.telegram_chat_id for user in self.users))

	def send_text(self, text: str) -> List[requests.Response]:
		"""Отправить текстовое сообщение всем пользователям с Telegram.
		
		Args:
			text: Текст сообщения
			
		Returns:
			List[requests.Response]: Список ответов от Telegram API
		"""
		if not self.enabled:
			return []
		url = f"https://api.telegram.org/bot{self.bot_token}/sendMessage"
		responses = []
		for user in self.users:
			if user.telegram_chat_id:
				payload = {"chat_id": user.telegram_chat_id, "text": text, "parse_mode": "HTML"}
				responses.append(requests.post(url, json=payload, timeout=20))
		return responses

	def send_document(self, caption: str, filename: str, content_bytes: bytes) -> List[requests.Response]:
		"""Отправить документ всем пользователям с Telegram.
		
		Args:
			caption: Подпись к документу
			filename: Имя файла
			content_bytes: Содержимое файла в байтах
			
		Returns:
			List[requests.Response]: Список ответов от Telegram API
		"""
		if not self.enabled:
			return []
		url = f"https://api.telegram.org/bot{self.bot_token}/sendDocument"
		responses = []
		for user in self.users:
			if user.telegram_chat_id:
				files = {"document": (filename, content_bytes)}
				data = {"chat_id": str(user.telegram_chat_id), "caption": caption, "parse_mode": "HTML"}
				responses.append(requests.post(url, data=data, files=files, timeout=30))
		return responses


class EmailTransport:
	"""Транспорт для отправки отчетов через SMTP."""
	
	def __init__(self, smtp_config: Optional[SmtpConfig], users: Optional[Iterable[NotificationUser]]) -> None:
		"""Инициализация Email транспорта.
		
		Args:
			smtp_config: Конфигурация SMTP сервера
			users: Список пользователей с настройками email
		"""
		self.smtp_config = smtp_config
		self.users = list(users) if users else []

	@property
	def enabled(self) -> bool:
		"""Проверить, настроен ли транспорт для отправки.
		
		Returns:
			bool: True если есть конфигурация и пользователи с email, False иначе
		"""
		return bool(self.smtp_config and any(user.email for user in self.users))

	def _connect(self) -> smtplib.SMTP:
		assert self.smtp_config is not None
		if self.smtp_config.use_ssl:
			context = ssl.create_default_context()
			server = smtplib.SMTP_SSL(self.smtp_config.host, self.smtp_config.port, context=context, timeout=30)
		else:
			server = smtplib.SMTP(self.smtp_config.host, self.smtp_config.port, timeout=30)
		if self.smtp_config.use_starttls and not self.smtp_config.use_ssl:
			server.starttls(context=ssl.create_default_context())
		server.login(self.smtp_config.username, self.smtp_config.password)
		return server

	def send(self, subject: str, body: str, attachments: Optional[List[tuple[str, bytes, str]]] = None) -> None:
		"""Отправить email всем пользователям с email адресами.
		
		Args:
			subject: Тема письма
			body: Текст письма
			attachments: Список вложений (filename, content_bytes, mime_type)
		"""
		if not self.enabled:
			return
		assert self.smtp_config is not None

		msg = EmailMessage()
		from_addr = self.smtp_config.from_addr or self.smtp_config.username
		msg["From"] = from_addr
		
		# Собираем всех пользователей с email
		email_recipients = [user.email for user in self.users if user.email]
		msg["To"] = ", ".join(email_recipients)
		msg["Subject"] = subject
		msg.set_content(body)

		for attachment in attachments or []:
			filename, content, mime_type = attachment
			maintype, subtype = mime_type.split("/", 1)
			msg.add_attachment(content, maintype=maintype, subtype=subtype, filename=filename)

		with self._connect() as server:
			server.send_message(msg)


from .manager import ErrorManager
from .email_config import SmtpConfig, NotificationUser
from .logger import get_global_logger, set_global_logger, PermanentContextLogger

__all__ = [
	"ErrorManager",
	"SmtpConfig",
	"NotificationUser",
	"get_global_logger",
	"set_global_logger",
	"PermanentContextLogger",
]

__version__ = "0.3.4"


from dataclasses import dataclass
from typing import Optional


@dataclass
class SmtpConfig:
	"""Конфигурация SMTP сервера для отправки email отчетов.
	
	Attributes:
		host: Адрес SMTP сервера
		port: Порт SMTP сервера
		username: Имя пользователя для аутентификации
		password: Пароль для аутентификации
		use_ssl: Использовать SSL соединение (по умолчанию True)
		from_addr: Email адрес отправителя (если None, используется username)
		use_starttls: Использовать STARTTLS для шифрования (взаимоисключающе с use_ssl)
	"""
	host: str
	port: int
	username: str
	password: str
	use_ssl: bool = True
	from_addr: Optional[str] = None
	use_starttls: bool = False


@dataclass
class NotificationUser:
	"""Пользователь для получения уведомлений.
	
	Attributes:
		name: Имя пользователя (для идентификации в логах)
		telegram_chat_id: ID чата Telegram (если None, уведомления в Telegram не отправляются)
		email: Email адрес (если None, уведомления на email не отправляются)
	"""
	name: str
	telegram_chat_id: Optional[int] = None
	email: Optional[str] = None


from __future__ import annotations

import logging
from contextlib import contextmanager
from typing import Iterable, Optional, Tuple

from .email_config import SmtpConfig, NotificationUser
from .logger import ErrorTrackingLogger, create_file_logger, set_global_logger
from .report import RunSummary, read_log_tail, build_report_text, build_log_attachment_bytes
from .transports import TelegramTransport, EmailTransport


PRIMARY_TELEGRAM = "telegram"
PRIMARY_EMAIL = "email"


class ErrorManager:
	"""Основной класс для управления логированием и отправкой отчетов.
	
	Создает файловый логгер, настраивает транспорты (Telegram/Email) и предоставляет
	методы для контекстного логирования и отправки отчетов по завершению работы.
	"""
	
	def __init__(
		self,
		log_file_path: str,
		telegram_bot_token: Optional[str] = None,
		users: Optional[Iterable[NotificationUser]] = None,
		smtp_config: Optional[SmtpConfig] = None,
		send_reports_without_errors: bool = False,
		primary_channel: str = PRIMARY_TELEGRAM,
		logger_name: str = "app",
		log_level: int = logging.INFO,
	) -> None:
		"""Инициализация менеджера ошибок.
		
		Args:
			log_file_path: Путь к файлу лога (папка создается автоматически)
			telegram_bot_token: Токен бота Telegram для отправки отчетов
			users: Список пользователей для получения уведомлений
			smtp_config: Конфигурация SMTP для отправки email
			send_reports_without_errors: Отправлять ли отчеты при отсутствии ошибок
			primary_channel: Приоритетный канал ("telegram" или "email")
			logger_name: Имя логгера в записях лога
			log_level: Уровень логирования (DEBUG, INFO, WARNING, ERROR, CRITICAL)
		"""
		self.log_file_path = log_file_path
		self.logger = create_file_logger(logger_name, log_file_path, level=log_level)
		set_global_logger(self.logger)
		self.tg = TelegramTransport(telegram_bot_token, users)
		self.mail = EmailTransport(smtp_config, users)
		self.send_reports_without_errors = send_reports_without_errors
		self.primary_channel = primary_channel.lower()
		self._active_run_name: Optional[str] = None

	def get_logger(self, run_name: Optional[str] = None) -> ErrorTrackingLogger:
		"""Получить логгер для записи сообщений.
		
		Args:
			run_name: Имя текущего запуска (используется в отчетах)
			
		Returns:
			ErrorTrackingLogger: Логгер с отслеживанием ошибок
		"""
		self._active_run_name = run_name
		return self.logger

	@contextmanager
	def error_context(self, name: str):
		"""Контекстный менеджер для пометки сообщений лога.
		
		Все сообщения внутри блока будут помечены указанным контекстом.
		
		Args:
			name: Имя контекста для пометки сообщений
			
		Yields:
			ErrorTrackingLogger: Логгер с активным контекстом
		"""
		with self.logger.context(name) as _:
			yield self.logger

	def send_report(self, run_name: Optional[str] = None) -> Tuple[bool, bool]:
		"""Отправить отчет о выполнении.
		
		Создает отчет с последними 300 строками лога и отправляет через настроенные
		транспорты (Telegram/Email) согласно приоритету каналов.
		
		Args:
			run_name: Имя запуска для отчета (если None, используется активное)
			
		Returns:
			Tuple[bool, bool]: (отправлено_в_telegram, отправлено_на_email)
		"""
		try:
			if self.logger.had_error or self.send_reports_without_errors:
				return self._send_report(run_name=run_name or self._active_run_name)
			return (False, False)
		except Exception as report_exc:
			logging.getLogger("runreporter.internal").error(f"Ошибка отправки отчета: {report_exc}")
			return (False, False)

	def _send_report(self, run_name: Optional[str]) -> Tuple[bool, bool]:
		log_tail = read_log_tail(self.log_file_path)
		summary = RunSummary(
			run_name=run_name,
			had_errors=self.logger.had_error,
			primary_channel=self.primary_channel,
			sent_to_telegram=False,
			sent_to_email=False,
		)
		text = build_report_text(summary, log_tail)
		attachment_bytes = build_log_attachment_bytes(log_tail)

		sent_tg = False
		sent_mail = False

		def try_send_telegram() -> bool:
			if not self.tg.enabled:
				return False
			self.tg.send_text(text)
			self.tg.send_document(caption="Лог последней сессии", filename="log_tail.txt", content_bytes=attachment_bytes)
			return True

		def try_send_email() -> bool:
			if not self.mail.enabled:
				return False
			self.mail.send(
				subject=f"Отчет выполнения: {run_name or ''}",
				body=text,
				attachments=[("log_tail.txt", attachment_bytes, "text/plain")],
			)
			return True

		# Priority sending
		if self.primary_channel == PRIMARY_TELEGRAM:
			sent_tg = try_send_telegram()
			sent_mail = try_send_email() if not sent_tg else try_send_email()
		elif self.primary_channel == PRIMARY_EMAIL:
			sent_mail = try_send_email()
			sent_tg = try_send_telegram() if not sent_mail else try_send_telegram()
		else:
			# fallback: attempt both
			sent_tg = try_send_telegram()
			sent_mail = try_send_email()

		return sent_tg, sent_mail

	@contextmanager
	def context(self, run_name: Optional[str] = None):
		"""Контекстный менеджер для автоматической отправки отчета.
		
		При выходе из контекста автоматически отправляет отчет, если были ошибки
		или включена отправка отчетов без ошибок.
		
		Args:
			run_name: Имя запуска для отчета
			
		Yields:
			ErrorTrackingLogger: Логгер для записи сообщений
		"""
		try:
			yield self.logger
		except Exception as exc:
			self.logger.exception(f"Исключение во время выполнения: {exc}")
			raise
		finally:
			try:
				if self.logger.had_error or self.send_reports_without_errors:
					self._send_report(run_name=run_name)
			except Exception as report_exc:
				# Avoid crashing the app due to reporting issues
				logging.getLogger("runreporter.internal").error(f"Ошибка отправки отчета: {report_exc}")


import logging
from contextlib import contextmanager
from typing import List, Optional
from logging import Logger
from pathlib import Path


_GLOBAL_LOGGER: Optional["ErrorTrackingLogger"] = None


class ErrorTrackingLogger:
	"""Обертка над Python logger с отслеживанием ошибок и контекстными метками.
	
	Автоматически отслеживает наличие ошибок в логах и поддерживает стек контекстов
	для пометки сообщений префиксами вида [Context1 > Context2].
	"""

	def __init__(self, logger: Logger) -> None:
		"""Инициализация логгера с отслеживанием ошибок.
		
		Args:
			logger: Базовый Python logger для записи сообщений
		"""
		self._logger = logger
		self._had_error = False
		self._context_stack: List[str] = []

	def _mark_error(self) -> None:
		self._had_error = True

	@property
	def had_error(self) -> bool:
		"""Проверить, были ли зафиксированы ошибки в логах.
		
		Returns:
			bool: True если были ошибки, False иначе
		"""
		return self._had_error

	def _with_ctx(self, msg: str) -> str:
		if not self._context_stack:
			return msg
		ctx = " > ".join(self._context_stack)
		return f"[{ctx}] {msg}"

	def debug(self, msg: str, *args, **kwargs) -> None:
		self._logger.debug(self._with_ctx(msg), *args, **kwargs)

	def info(self, msg: str, *args, **kwargs) -> None:
		self._logger.info(self._with_ctx(msg), *args, **kwargs)

	def warning(self, msg: str, *args, **kwargs) -> None:
		self._logger.warning(self._with_ctx(msg), *args, **kwargs)

	def error(self, msg: str, *args, **kwargs) -> None:
		self._mark_error()
		self._logger.error(self._with_ctx(msg), *args, **kwargs)

	def exception(self, msg: str, *args, exc_info: bool = True, **kwargs) -> None:
		self._mark_error()
		self._logger.error(self._with_ctx(msg), *args, exc_info=exc_info, **kwargs)

	def critical(self, msg: str, *args, **kwargs) -> None:
		self._mark_error()
		self._logger.critical(self._with_ctx(msg), *args, **kwargs)

	@contextmanager
	def context(self, name: str):
		"""Контекстный менеджер для пометки сообщений.
		
		Все сообщения внутри блока будут помечены указанным контекстом.
		Контексты могут быть вложенными.
		
		Args:
			name: Имя контекста для пометки сообщений
			
		Yields:
			ErrorTrackingLogger: Текущий логгер с активным контекстом
		"""
		self._context_stack.append(str(name))
		try:
			yield self
		finally:
			self._context_stack.pop()

	def with_permanent_context(self, context_name: str) -> "PermanentContextLogger":
		"""Создать логгер с постоянным контекстом.
		
		Все сообщения будут автоматически помечены указанным контекстом.
		
		Args:
			context_name: Имя контекста для постоянной пометки сообщений
			
		Returns:
			PermanentContextLogger: Логгер с постоянным контекстом
		"""
		return PermanentContextLogger(self, context_name)


class PermanentContextLogger:
	"""Логгер с постоянным контекстом модуля.
	
	Все сообщения автоматически помечаются указанным контекстом.
	"""

	def __init__(self, base: ErrorTrackingLogger, context_name: str) -> None:
		"""Инициализация логгера с постоянным контекстом.
		
		Args:
			base: Базовый логгер с отслеживанием ошибок
			context_name: Имя контекста для постоянной пометки
		"""
		self._base = base
		self._context = str(context_name)

	@property
	def had_error(self) -> bool:
		"""Проверить, были ли зафиксированы ошибки в базовом логгере.
		
		Returns:
			bool: True если были ошибки, False иначе
		"""
		return self._base.had_error

	def _p(self, msg: str) -> str:
		return f"[{self._context}] {msg}"

	def debug(self, msg: str, *args, **kwargs) -> None:
		self._base.debug(self._p(msg), *args, **kwargs)

	def info(self, msg: str, *args, **kwargs) -> None:
		self._base.info(self._p(msg), *args, **kwargs)

	def warning(self, msg: str, *args, **kwargs) -> None:
		self._base.warning(self._p(msg), *args, **kwargs)

	def error(self, msg: str, *args, **kwargs) -> None:
		self._base.error(self._p(msg), *args, **kwargs)

	def exception(self, msg: str, *args, **kwargs) -> None:
		self._base.exception(self._p(msg), *args, **kwargs)

	def critical(self, msg: str, *args, **kwargs) -> None:
		self._base.critical(self._p(msg), *args, **kwargs)

	@contextmanager
	def context(self, name: str):
		"""Контекстный менеджер для дополнительных контекстов.
		
		Сообщения будут помечены как [ModuleContext > AdditionalContext].
		
		Args:
			name: Имя дополнительного контекста
			
		Yields:
			PermanentContextLogger: Текущий логгер с дополнительным контекстом
		"""
		with self._base.context(f"{self._context} > {name}") as _:
			yield self


class ComponentLogger:
	"""Легковесная обертка для логгера с префиксом компонента.
	
	Все сообщения автоматически помечаются префиксом [ComponentName].
	"""

	def __init__(self, base: ErrorTrackingLogger, component_name: str) -> None:
		"""Инициализация логгера компонента.
		
		Args:
			base: Базовый логгер с отслеживанием ошибок
			component_name: Имя компонента для префикса
		"""
		self._base = base
		self._component = str(component_name)

	@property
	def had_error(self) -> bool:
		"""Проверить, были ли зафиксированы ошибки в базовом логгере.
		
		Returns:
			bool: True если были ошибки, False иначе
		"""
		return self._base.had_error

	def _p(self, msg: str) -> str:
		return f"[{self._component}] {msg}"

	def debug(self, msg: str, *args, **kwargs) -> None:
		self._base.debug(self._p(msg), *args, **kwargs)

	def info(self, msg: str, *args, **kwargs) -> None:
		self._base.info(self._p(msg), *args, **kwargs)

	def warning(self, msg: str, *args, **kwargs) -> None:
		self._base.warning(self._p(msg), *args, **kwargs)

	def error(self, msg: str, *args, **kwargs) -> None:
		self._base.error(self._p(msg), *args, **kwargs)

	def exception(self, msg: str, *args, **kwargs) -> None:
		self._base.exception(self._p(msg), *args, **kwargs)

	def critical(self, msg: str, *args, **kwargs) -> None:
		self._base.critical(self._p(msg), *args, **kwargs)

	@contextmanager
	def context(self, name: str):
		"""Контекстный менеджер для пометки сообщений компонента.
		
		Сообщения будут помечены как [Component > Context].
		
		Args:
			name: Имя контекста
			
		Yields:
			ComponentLogger: Текущий логгер с активным контекстом
		"""
		with self._base.context(f"{self._component} > {name}") as _:
			yield self


def set_global_logger(logger: ErrorTrackingLogger) -> None:
	"""Установить глобальный логгер для использования в модулях.
	
	Args:
		logger: Логгер для установки как глобальный
	"""
	global _GLOBAL_LOGGER
	_GLOBAL_LOGGER = logger


def get_global_logger() -> ErrorTrackingLogger:
	"""Получить глобальный логгер.
	
	Returns:
		ErrorTrackingLogger: Глобальный логгер
		
	Raises:
		RuntimeError: Если глобальный логгер не установлен
	"""
	if _GLOBAL_LOGGER is None:
		raise RuntimeError("Global logger is not set. Initialize ErrorManager first or call set_global_logger().")
	return _GLOBAL_LOGGER


def get_logger_for(component_name: str) -> ComponentLogger:
	"""Получить логгер для компонента с префиксом.
	
	Args:
		component_name: Имя компонента для префикса
		
	Returns:
		ComponentLogger: Логгер с префиксом компонента
	"""
	return get_global_logger().with_component(component_name)


def create_file_logger(name: str, log_file_path: str, level: int = logging.INFO) -> ErrorTrackingLogger:
	# Ensure log directory exists
	path = Path(log_file_path)
	if path.parent and not path.parent.exists():
		path.parent.mkdir(parents=True, exist_ok=True)

	logger = logging.getLogger(name)
	logger.setLevel(level)
	logger.propagate = False

	# Avoid duplicate handlers if called multiple times
	if not any(isinstance(h, logging.FileHandler) and getattr(h, 'baseFilename', None) == str(path) for h in logger.handlers):
		file_handler = logging.FileHandler(str(path), encoding="utf-8")
		formatter = logging.Formatter("%(asctime)s [%(levelname)s] %(name)s: %(message)s")
		file_handler.setFormatter(formatter)
		logger.addHandler(file_handler)

	return ErrorTrackingLogger(logger)


from __future__ import annotations

from dataclasses import dataclass
from pathlib import Path
from typing import Optional


DEFAULT_MAX_TAIL_LINES = 300


@dataclass
class RunSummary:
	"""Сводка о выполнении задачи для отчета.
	
	Attributes:
		run_name: Имя задачи
		had_errors: Были ли ошибки во время выполнения
		primary_channel: Приоритетный канал отправки
		sent_to_telegram: Отправлен ли отчет в Telegram
		sent_to_email: Отправлен ли отчет на email
	"""
	run_name: Optional[str]
	had_errors: bool
	primary_channel: str
	sent_to_telegram: bool
	sent_to_email: bool

	def to_text(self) -> str:
		"""Преобразовать сводку в текстовый формат для отчета.
		
		Returns:
			str: Текстовое представление сводки
		"""
		name_part = f"Имя задачи: {self.run_name}\n" if self.run_name else ""
		status = "С ошибками" if self.had_errors else "Без ошибок"
		primary = f"Приоритетный канал: {self.primary_channel}\n"
		channels = f"Отправлено: Telegram={self.sent_to_telegram}, Email={self.sent_to_email}\n"
		return f"Отчет выполнения\n{name_part}Статус: {status}\n{primary}{channels}"


def read_log_tail(log_file_path: str, max_lines: int = DEFAULT_MAX_TAIL_LINES) -> str:
	path = Path(log_file_path)
	if not path.exists():
		return "Лог-файл отсутствует."
	# Efficient tail read
	with path.open("r", encoding="utf-8", errors="ignore") as f:
		lines = f.readlines()
		return "".join(lines[-max_lines:])


def build_report_text(summary: RunSummary, log_tail: str) -> str:
	return (
		f"{summary.to_text()}\n"
		"Последние строки лога (до 300):\n"
		"-------------------------------\n"
		f"{log_tail}"
	)


def build_log_attachment_bytes(log_tail: str) -> bytes:
	return log_tail.encode("utf-8", errors="ignore")